﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using System.Collections.ObjectModel;
using System.Collections.Specialized;

namespace _367268_TabControlSample
{
	public class MainViewModel : ViewModelBase
    {
        #region Fields

        private TabViewModel selectedTab;
        private ObservableCollection<TabViewModel> tabs;
        
        #endregion

        #region Properties
        
        public TabViewModel SelectedTab
		{
			get
			{
				return this.selectedTab;
			}
			set
			{
				if (this.selectedTab != value)
				{
					this.selectedTab = value;
					OnPropertyChanged("SelectedTab");
                    this.RefreshItems();
				}
			}
		}
        public ObservableCollection<TabViewModel> Tabs
        {
            get
            {
                if (this.tabs == null)
                {
                    this.tabs = new ObservableCollection<TabViewModel>();
                    this.tabs.CollectionChanged += new NotifyCollectionChangedEventHandler(OnTabsCollectionChanged);
                }
                return this.tabs;
            }
            set
            {
                if (this.tabs != value)
                {
                    this.tabs = value;
                    OnPropertyChanged("Tabs");
                }
            }
        }
        public DelegateCommand AddNewTab { get; set; }

        #endregion

        public MainViewModel()
        {
            this.AddNewTab = new DelegateCommand((p) =>
            {
                this.InsertNewTab();
            });
            this.AddItems(1);
            this.SelectedTab = this.Tabs[0];
        }

        private void OnTabsCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (TabViewModel tab in e.NewItems)
                    {
                        tab.Remove += new EventHandler(OnItemRemove);
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (TabViewModel tab in e.OldItems)
                    {
                        tab.Remove -= new EventHandler(OnItemRemove);
                    }
                    break;
                case NotifyCollectionChangedAction.Replace:
                    break;
                case NotifyCollectionChangedAction.Reset:
                    break;
                default:
                    break;
            }
        }
        private void OnItemRemove(object sender, EventArgs e)
        {
            if (this.tabs.Count > 1)
            {
                this.Tabs.Remove(sender as TabViewModel);
            }
        }
        private void RefreshItems()
        {
            foreach (var item in this.Tabs)
            {
                item.RemoveTab.InvalidateCanExecute();
            }
        }
		private void AddItems(int count)
		{
			for (int i = 0; i < count; i++)
			{
                this.Tabs.Add(this.CreateNewItem());
			}
		}
        private TabViewModel CreateNewItem()
        {
            string header = "Item " + (this.Tabs.Count + 1);
            return new TabViewModel() { Header = header, Url=@"http:\\www.something.com\" + header };
        }
        private void InsertNewTab()
        {
            TabViewModel model = this.CreateNewItem();
            this.Tabs.Add(model);
            this.SelectedTab = model;
        }
	}

	public class TabViewModel : ViewModelBase
    {
        #region Fields
        
        private bool isSelected;
        private string url;
        private bool isMouseOver;
        private bool isDeleteAllowed;

        #endregion

        #region Properties

        public string Header { get; set; }

        public bool IsSelected
        {
            get { return isSelected; }
            set
            {
                isSelected = value;
                this.OnPropertyChanged("IsSelected");
                this.OnPropertyChanged("IsDeleteAllowed");
            }
        }

        public bool IsMouseOver
        {
            get { return isMouseOver; }
            set
            {
                isMouseOver = value;
                this.OnPropertyChanged("IsMouseOver");
                this.OnPropertyChanged("IsDeleteAllowed");
            }
        }

        public bool IsDeleteAllowed
        {
            get { return this.IsSelected || this.IsMouseOver; }
        }
        
        public string Url
        {
            get { return url; }
            set
            {
                url = value;
                this.OnPropertyChanged("Url");
            }
        }

        public DelegateCommand RemoveTab { get; set; }

        #endregion

        public event EventHandler Remove;

        public TabViewModel()
        {
            this.RemoveTab = new DelegateCommand((p) =>
                {
                    this.OnRemove(this, EventArgs.Empty);
                });
        }

        protected virtual void OnRemove(object sender, EventArgs e)
        {
            if (this.Remove != null)
            {
                this.Remove(this, EventArgs.Empty);
            }
        }
	}
}
